;*************************************************************************
; Copyright (c) 2002 The University of Chicago, as Operator of Argonne
; National Laboratory.
; Copyright (c) 2002 The Regents of the University of California, as
; Operator of Los Alamos National Laboratory.
; This file is distributed subject to a Software License Agreement found
; in the file LICENSE that is included with this distribution. 
;*************************************************************************
; the following procedure plots rawImage at 1:1
PRO DRAW_IMAGE

  COMMON CATCH_IMAGE, widget_ids, imageData
  COMMON DRAW10_Comm, DRAW10_Id, TEXT8_Id, selRectBGOUP_Id, calPointEntry

  WSET, DRAW10_Id
  ERASE

  ; Rotate if specified. 
  selectedImage = ROTATE(imageData.rawImage, imageData.calTemp.rotation)

  IF(imageData.scaleINT EQ 0) THEN $
      TV, selectedImage  $
  ELSE $
      TVSCL, selectedImage

 
  ;superimpose CAL rectangle 
  xmin = imageData.calTemp.xmin_PIX
  ymin = imageData.calTemp.ymin_PIX
  xmax = imageData.calTemp.xmax_PIX
  ymax = imageData.calTemp.ymax_PIX
; PLOTS, xmin, ymin, LineStyle=2, COLOR = 254, /DEVICE
; PLOTS, xmax, ymin, /CONTINUE, LineStyle=2, COLOR = 254, /DEVICE
; PLOTS, xmax, ymax, /CONTINUE, LineStyle=2, COLOR = 254, /DEVICE
; PLOTS, xmin, ymax, /CONTINUE, LineStyle=2, COLOR = 254, /DEVICE
; PLOTS, xmin, ymin, /CONTINUE, LineStyle=2, COLOR = 254, /DEVICE
  PLOTS, imageData.calTemp.bl_x_PIX, imageData.calTemp.bl_y_PIX, $
         LineStyle=2, COLOR = 254, /DEVICE
  PLOTS, imageData.calTemp.br_x_PIX, imageData.calTemp.br_y_PIX, $
         /CONTINUE, LineStyle=2, COLOR = 254, /DEVICE
  PLOTS, imageData.calTemp.tr_x_PIX, imageData.calTemp.tr_y_PIX, $
         /CONTINUE, LineStyle=2, COLOR = 254, /DEVICE
  PLOTS, imageData.calTemp.tl_x_PIX, imageData.calTemp.tl_y_PIX, $
         /CONTINUE, LineStyle=2, COLOR = 254, /DEVICE
  PLOTS, imageData.calTemp.bl_x_PIX, imageData.calTemp.bl_y_PIX, $
         /CONTINUE, LineStyle=2, COLOR = 254, /DEVICE

  IF(imageData.warp) THEN BEGIN
      ;read image back in, including cal trapezoid 
      readback = TVRD(0, 0, imageData.xsize, imageData.ysize)
      xi = [imageData.calTemp.bl_x_PIX, $
            imageData.calTemp.tl_x_PIX, $
            imageData.calTemp.tr_x_PIX, $
            imageData.calTemp.br_x_PIX]

      yi = [imageData.calTemp.bl_y_PIX, $
            imageData.calTemp.tl_y_PIX, $
            imageData.calTemp.tr_y_PIX, $
            imageData.calTemp.br_y_PIX]

      warped_tr_x_PIX = imageData.calTemp.tl_x_PIX +   $
            ((imageData.calTemp.tr_x_PIX - imageData.calTemp.tl_x_PIX)/.707)
      warped_br_x_PIX = warped_tr_x_PIX

      xo = [imageData.calTemp.bl_x_PIX, $
            imageData.calTemp.tl_x_PIX, $
            warped_tr_x_PIX, $
            warped_br_x_PIX]

      warped_tr_y_PIX = imageData.calTemp.tl_y_PIX
      warped_br_y_PIX = imageData.calTemp.bl_y_PIX

      yo = [imageData.calTemp.bl_y_PIX, $
            imageData.calTemp.tl_y_PIX, $
            warped_tr_y_PIX, $
            warped_br_y_PIX]

      selectedImage = WARP_TRI(xo, yo, xi, yi, readback)
 
      ERASE
      IF(imageData.scaleINT EQ 0) THEN $
          TV, selectedImage  $
      ELSE $
          TVSCL, selectedImage
  END

  ;superimpose ROI rectangle 
  xmin = imageData.calTemp.xmin_ROI
  ymin = imageData.calTemp.ymin_ROI
  xmax = imageData.calTemp.xmax_ROI
  ymax = imageData.calTemp.ymax_ROI
  PLOTS, xmin, ymin, LineStyle=0, COLOR = 254, /DEVICE
  PLOTS, xmax, ymin, /CONTINUE, LineStyle=0, COLOR = 254, /DEVICE
  PLOTS, xmax, ymax, /CONTINUE, LineStyle=0, COLOR = 254, /DEVICE
  PLOTS, xmin, ymax, /CONTINUE, LineStyle=0, COLOR = 254, /DEVICE
  PLOTS, xmin, ymin, /CONTINUE, LineStyle=0, COLOR = 254, /DEVICE

  XYOUTS, 4, 4, imageData.calTemp.desc, /DEVICE, COLOR = 255, Size=2.0

  rotateLabel = ['0', '90', '180', '270']
  str1 = 'Index ' + STRTRIM(STRING(imageData.screen_id),2) + ' / ' + $
         STRTRIM(imageData.calTemp.desc,2) + ' / ' + $
         'Rotation = ' +  rotateLabel[imageData.calTemp.rotation] 
 
  IF(imageData.calTemp.calibrated NE 0) THEN BEGIN
      str1 = str1 + '  CALIBRATED to ' + imageData.calTemp.egu
  ENDIF ELSE BEGIN
      str1 = str1 + '  UNCALIBRATED to Pixels'
  ENDELSE
  WIDGET_CONTROL, TEXT8_Id, SET_VALUE=str1

END


PRO CALC_EGU_ARRAYS

  COMMON CATCH_IMAGE, widget_ids, imageData

    IF (imageData.cal.calibrated NE 0) THEN BEGIN
      xGain   = (imageData.cal.xmax_EGU - imageData.cal.xmin_EGU) / $
                (FLOAT(imageData.cal.xmax_PIX - imageData.cal.xmin_PIX))
      yGain   = (imageData.cal.ymax_EGU - imageData.cal.ymin_EGU) / $
                (FLOAT(imageData.cal.ymax_PIX - imageData.cal.ymin_PIX))

      xOffset=imageData.cal.xmin_EGU - (imageData.cal.xmin_PIX * xGain)
      yOffset=imageData.cal.ymin_EGU - (imageData.cal.ymin_PIX * yGain)

      imageData.x_axisEGU = (FINDGEN(imageData.xsize) * xGain) + xOffset
      imageData.y_axisEGU = (FINDGEN(imageData.ysize) * yGain) + yOffset
      imageData.xOffset = xOffset
      imageData.xGain = xGain
      imageData.yOffset = yOffset
      imageData.yGain = yGain
    ENDIF ELSE BEGIN
      imageData.x_axisEGU = FINDGEN(imageData.xsize)
      imageData.y_axisEGU = FINDGEN(imageData.ysize)
      imageData.xOffset = 0.0
      imageData.xGain = 1.0
      imageData.yOffset = 0.0
      imageData.yGain = 1.0
    ENDELSE

END
  
PRO calFilePDMENU_Event, Event

  COMMON CATCH_IMAGE, widget_ids, imageData, screenCalArray
  COMMON DRAW10_Comm, DRAW10_Id, TEXT8_Id, selRectBGROUP_Id, calPointEntry

  CASE Event.Value OF 


  'File.Open ...': BEGIN
    PRINT, 'Event for File.Open ...'
        F=DIALOG_PICKFILE(/READ, GET_PATH=P, FILTER = imageData.lpath + '*.cal')
        IF F eq '' THEN return
        IF (STRMID(F,0,1) EQ '/') THEN $
                FNAME = F $
        ELSE $
                FNAME = P+F

        Print, FNAME

        IF (FILE_EXISTS(FNAME) EQ 0) THEN BEGIN
            response = WIDGET_MESSAGE('File does not exist !')
            RETURN
        ENDIF

        ; PrintF, lun, screenCalArray did not separate structures with CR's
        ; so I guess I'm stuck with formatting it myself  :(

        calInfo = imageData.cal

        OpenR, lun, FNAME, /Get_LUN
        desc = '          '
        vers = 0L  
        ReadF, lun, desc, vers, FORMAT = '((A10), (I))'
        IF(vers EQ imageData.calFileVer) THEN BEGIN
            FOR I = 0, 100 DO BEGIN
                ReadF, lun, calInfo, FORMAT = imageData.calFormat
                screenCalArray[I] = calInfo
            END
        ENDIF ELSE BEGIN
            WSET, DRAW10_Id
            str1 = 'CALIBRATION FILE VERSION ERROR'
            XYOUTS, 10, 180, str1, /DEVICE, COLOR = 255, SIZE = 1.7
        ENDELSE

        Free_LUN, lun
    END

  'File.Save ...': BEGIN
    PRINT, 'Event for File.Save ...'
        F=DIALOG_PICKFILE(/WRITE, GET_PATH=P, FILTER = imageData.lpath + '*.cal')
        IF F eq '' THEN return

        IF (STRMID(F,STRLEN(F)-4, 4) NE '.cal') THEN $
                F = F + '.cal'

        IF (STRMID(F,0,1) EQ '/') THEN $
                FNAME = F $
        ELSE $
                FNAME = P+F

        Print, FNAME
 
        IF (FILE_EXISTS(FNAME) EQ 1) THEN BEGIN
            response = WIDGET_MESSAGE('Overwrite existing file ?', /QUESTION)
            IF (response EQ 'No') THEN RETURN
        ENDIF

        ; PrintF, lun, screenCalArray did not separate structures with CR's
        ; so I guess I'm stuck with formatting it myself  :(

        OpenW, lun, FNAME,  /Get_LUN, Error = err
   
        IF (err NE 0)  THEN BEGIN
            result = WIDGET_MESSAGE("FILE WRITE ERROR !")
        ENDIF ELSE BEGIN
            ;write version
            PrintF, lun, imageData.calFileVer, FORMAT = '("Version ",I)' 

            FOR I = 0, 100 DO BEGIN
                PrintF, lun, screenCalArray[I], FORMAT = imageData.calFormat
            END
            Free_LUN, lun
        ENDELSE
    END

  'File.Quit': BEGIN
    PRINT, 'Event for File.Quit'
    WIDGET_CONTROL, event.top, /DESTROY
    END
  ENDCASE
END

;*************  Widget Event Routine  *************
;*************  WIDGET EVENT ROUTINE  *************
;
;
; Called by the XMANAGER when a widget event occurs
;
;
;**************************************************

PRO MAIN13_Event, Event

  COMMON DRAW10_Comm, DRAW10_Id, TEXT8_Id, selRectBGROUP_Id, calPointEntry
  COMMON CATCH_IMAGE, widget_ids, imageData, screenCalArray

  WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev

  CASE Ev OF 

  ; Event for File
  'calFilePDMENU': calFilePDMENU_Event, Event

  ; Event for rotPDMENU
  'rotPDMENU': BEGIN                  
       CASE Event.Value OF
           'Rotation.0': BEGIN
                PRINT, 'Event for Rotation.0'
                imageData.calTemp.rotation = 0
             END
           'Rotation.90': BEGIN
                PRINT, 'Event for Rotation.90'
                imageData.calTemp.rotation = 1
             END
           'Rotation.180': BEGIN
                PRINT, 'Event for Rotation.180'
                imageData.calTemp.rotation = 2
             END
           'Rotation.270': BEGIN
                PRINT, 'Event for Rotation.270'
                imageData.calTemp.rotation = 3
             END
       ENDCASE

       DRAW_IMAGE
   END


  ; Event for updateBUTTON
  'updateBUTTON': BEGIN
      Print, 'Event for Update Image'
      DRAW_IMAGE
      END

  ; Event for setCalBUTTON
  'setCalBUTTON': BEGIN
      Print, 'Event for set CAL'

          ;Determine if 'warping' is needed 
          IF((imageData.calTemp.tl_y_PIX EQ imageData.calTemp.tr_y_PIX) $
          AND (imageData.calTemp.bl_y_PIX EQ imageData.calTemp.br_y_PIX)) THEN $
              imageData.calTemp.calibrated = 1 $
          ELSE $
    ;         disable warping for now
              imageData.calTemp.calibrated = 1
    ;         imageData.calTemp.calibrated = 2

          imageData.calTemp.xmin_PIX = imageData.calTemp.tl_x_PIX

          ; If warped, adjust xmax 
          IF(imageData.calTemp.calibrated EQ 2) THEN $
              imageData.calTemp.xmax_PIX = imageData.calTemp.tl_x_PIX +  $
              ((imageData.calTemp.tr_x_PIX - imageData.calTemp.tl_x_PIX)/.707) $
          ELSE $
              imageData.calTemp.xmax_PIX = imageData.calTemp.tr_x_PIX

          imageData.calTemp.ymin_PIX = imageData.calTemp.br_y_PIX
          imageData.calTemp.ymax_PIX = imageData.calTemp.tl_y_PIX
          imageData.calTemp.xmin_EGU = imageData.calTemp.tl_x_EGU
          imageData.calTemp.xmax_EGU = imageData.calTemp.br_x_EGU
          imageData.calTemp.ymin_EGU = imageData.calTemp.br_y_EGU
          imageData.calTemp.ymax_EGU = imageData.calTemp.tl_y_EGU

          screenCalArray[imageData.screen_id] = imageData.calTemp
          imageData.cal = imageData.calTemp
          CALC_EGU_ARRAYS
          DRAW_IMAGE
          fakeEvent = {ID: Event.Id,  $
                       TOP: Event.Id, $
                       HANDLER: 0,    $
                       VALUE: "File.Save ..."}
          calFilePDMENU_Event, fakeEvent       ; Prompt for Save

   END

  ; Event for setUnCalBUTTON
  'setUnCalBUTTON': BEGIN
      Print, 'Event for set UnCal'
         imageData.calTemp.calibrated = 0
         imageData.calTemp.xmin_PIX = 2L
         imageData.calTemp.xmax_PIX = FIX(imageData.xsize-3)
         imageData.calTemp.ymin_PIX = 2L
         imageData.calTemp.ymax_PIX = FIX(imageData.ysize-3)
         imageData.x_axisEGU = FINDGEN(imageData.xsize)
         imageData.y_axisEGU = FINDGEN(imageData.ysize)
         imageData.xOffset = 0.0
         imageData.xGain = 1.0
         imageData.yOffset = 0.0
         imageData.yGain = 1.0

         screenCalArray[imageData.screen_id] = imageData.calTemp
         imageData.cal = imageData.calTemp
         DRAW_IMAGE

         fakeEvent = {ID: Event.Id,  $
                      TOP: Event.Id, $
                      HANDLER: 0,    $
                      VALUE: "File.Save ..."}
         calFilePDMENU_Event, fakeEvent       ; Prompt for Save
   END

  'TEXT8': BEGIN
      Print, 'Event for TEXT8'
   END

  'selRectBGROUP': BEGIN
       Print, 'Event for selRectBGROUP'
   END

  'DRAW10': BEGIN
      Print, 'Event for DRAW10'
      Print, 'X=', Event.X, ' Y= ', Event.Y, 'Press=',Event.TYPE
      IF (calPointEntry NE 0) THEN BEGIN
        IF (Event.PRESS EQ 1)  THEN BEGIN
            CASE calPointEntry of 
                1: BEGIN
                       imageData.calTemp.tl_x_PIX = Event.X
                       imageData.calTemp.tl_y_PIX = Event.Y
                       ; Force a trapezoid, for now
                       imageData.calTemp.bl_x_PIX = Event.X
                   END
                2: BEGIN
                       imageData.calTemp.bl_x_PIX = Event.X
                       imageData.calTemp.bl_y_PIX = Event.Y
                       ; Force a trapezoid, for now
                       imageData.calTemp.tl_x_PIX = Event.X
                   END
                3: BEGIN
                       imageData.calTemp.tr_x_PIX = Event.X
                       imageData.calTemp.tr_y_PIX = Event.Y
                       ; Force a trapezoid, for now
                       imageData.calTemp.br_x_PIX = Event.X
                   END
                4: BEGIN
                       imageData.calTemp.br_x_PIX = Event.X
                       imageData.calTemp.br_y_PIX = Event.Y
                       ; Force a trapezoid, for now
                       imageData.calTemp.tr_x_PIX = Event.X
                   END
            ENDCASE
           
            calPointEntry = 0
            DRAW_IMAGE
         ENDIF
         RETURN
      ENDIF
      IF (Event.PRESS EQ 1)  THEN BEGIN
        WIDGET_CONTROL, Event.Id, SENSITIVE = 0
    
        WSET, DRAW10_Id

        WIDGET_CONTROL, selRectBGROUP_Id, GET_VALUE = box
        
        IF(box EQ 0) THEN BEGIN
            x  = FIX(imageData.calTemp.tl_x_PIX)
            xs = FIX(imageData.calTemp.tr_x_PIX - imageData.calTemp.tl_x_PIX)
            y  = FIX(imageData.calTemp.bl_y_PIX)
            ys = FIX(imageData.calTemp.tl_y_PIX - imageData.calTemp.bl_y_PIX)
        ENDIF ELSE BEGIN
            x  = FIX(imageData.calTemp.xmin_ROI)
            xs = FIX(imageData.calTemp.xmax_ROI - imageData.calTemp.xmin_ROI)
            y  = FIX(imageData.calTemp.ymin_ROI)
            ys = FIX(imageData.calTemp.ymax_ROI - imageData.calTemp.ymin_ROI)
        ENDELSE

        MY_BOX_CURSOR,x,y,xs,ys, /INIT
        Print, x,y,xs,ys
        x = (x > 0) < 500
        y = (y > 0) < 480
        xs = xs > 5
        ys = ys > 5
      
        IF(box EQ 0) THEN BEGIN
            imageData.calTemp.tl_x_PIX = x
            imageData.calTemp.bl_x_PIX = x
            imageData.calTemp.tr_x_PIX = x + xs
            imageData.calTemp.br_x_PIX = x + xs
            imageData.calTemp.bl_y_PIX = y
            imageData.calTemp.br_y_PIX = y
            imageData.calTemp.tl_y_PIX = y + ys
            imageData.calTemp.tr_y_PIX = y + ys
        ENDIF ELSE BEGIN
            imageData.calTemp.xmin_ROI = x
            imageData.calTemp.xmax_ROI = x + xs
            imageData.calTemp.ymin_ROI = y
            imageData.calTemp.ymax_ROI = y + ys
        ENDELSE

        WAIT, .2
        WIDGET_CONTROL, Event.Id, SENSITIVE = 1

        DRAW_IMAGE
      ENDIF 
      END

  'TL_X_FIELD': BEGIN
      Print, 'Event for T/L X:'
        imageData.calTemp.tl_x_EGU = Event.Value
        imageData.calTemp.bl_x_EGU = Event.Value
        UPDATE_COORD
        DRAW_IMAGE
      END

  'TL_Y_FIELD': BEGIN
      Print, 'Event for T/L Y:'
        imageData.calTemp.tl_y_EGU = Event.Value
        imageData.calTemp.tr_y_EGU = Event.Value
        UPDATE_COORD
        DRAW_IMAGE
      END

  'BL_X_FIELD': BEGIN
      Print, 'Event for B/L X:'
        imageData.calTemp.bl_x_EGU = Event.Value
        imageData.calTemp.tl_x_EGU = Event.Value
        UPDATE_COORD
        DRAW_IMAGE
      END

  'BL_Y_FIELD': BEGIN
      Print, 'Event for B/L Y:'
        imageData.calTemp.bl_y_EGU = Event.Value
        imageData.calTemp.br_y_EGU = Event.Value
        UPDATE_COORD
        DRAW_IMAGE
      END

  'TR_X_FIELD': BEGIN
      Print, 'Event for T/R X:'
        imageData.calTemp.tr_x_EGU = Event.Value
        imageData.calTemp.br_x_EGU = Event.Value
        UPDATE_COORD
        DRAW_IMAGE
      END

  'TR_Y_FIELD': BEGIN
      Print, 'Event for T/R Y:'
        imageData.calTemp.tr_y_EGU = Event.Value
        imageData.calTemp.tl_y_EGU = Event.Value
        UPDATE_COORD
        DRAW_IMAGE
      END

  'BR_X_FIELD': BEGIN
      Print, 'Event for B/R X:'
        imageData.calTemp.br_x_EGU = Event.Value
        imageData.calTemp.tr_x_EGU = Event.Value
        UPDATE_COORD
        DRAW_IMAGE
      END

  'BR_Y_FIELD': BEGIN
      Print, 'Event for B/R Y:'
        imageData.calTemp.br_y_EGU = Event.Value
        imageData.calTemp.bl_y_EGU = Event.Value
        UPDATE_COORD
        DRAW_IMAGE
      END

  'TL_BUTTON':BEGIN
      Print, 'Event for TL Set'
        IF(calPointEntry EQ 0) THEN BEGIN
            calPointEntry = 1
            XYOUTS, 50, 460, 'LMB on T/L Corner or SET again to abort', $
                    /DEVICE, COLOR = 255, Size=2.0
        ENDIF ELSE BEGIN
            calPointEntry = 0  ;Abort set point
            DRAW_IMAGE
        ENDELSE

      END

  'BL_BUTTON':BEGIN
      Print, 'Event for BL Set'
        IF(calPointEntry EQ 0) THEN BEGIN
            calPointEntry = 2
            XYOUTS, 50, 460, 'LMB on B/L Corner or SET again to abort', $
                    /DEVICE, COLOR = 255, Size=2.0
        ENDIF ELSE BEGIN
            calPointEntry = 0  ;Abort set point
            DRAW_IMAGE
        ENDELSE

      END

  'TR_BUTTON':BEGIN
      Print, 'Event for TR Set'
        IF(calPointEntry EQ 0) THEN BEGIN
            calPointEntry = 3
            XYOUTS, 50, 460, 'LMB on T/R Corner or SET again to abort', $
                    /DEVICE, COLOR = 255, Size=2.0
        ENDIF ELSE BEGIN
            calPointEntry = 0  ;Abort set point
            DRAW_IMAGE
        ENDELSE

      END

  'BR_BUTTON':BEGIN
      Print, 'Event for BR Set'
        IF(calPointEntry EQ 0) THEN BEGIN
            calPointEntry = 4
            XYOUTS, 50, 460, 'LMB on B/R Corner or SET again to abort', $
                    /DEVICE, COLOR = 255, Size=2.0
        ENDIF ELSE BEGIN
            calPointEntry = 0  ;Abort set point
            DRAW_IMAGE
        ENDELSE

      END

  'plotChoiceBGROUP':BEGIN
      Print, 'Event for Warp', Event.Select
        imageData.warp = Event.Select
        DRAW_IMAGE
      END

  'eguFIELD': BEGIN
      Print, 'Event for EGU ', Event.Value
        imageData.calTemp.egu = Event.Value
        DRAW_IMAGE
      END

  'descFIELD': BEGIN
      Print, 'Event for DESC ', Event.Value
        imageData.calTemp.desc = Event.Value
        DRAW_IMAGE
      END

  ENDCASE
END

;********************** Main Program ********************
PRO screen_cal, GROUP=Group

  COMMON CATCH_IMAGE, widget_ids, imageData

  IF N_ELEMENTS(Group) EQ 0 THEN GROUP=0

  junk   = { CW_PDMENU_S, flags:0, name:'' }

  imageData.calTemp = imageData.cal

  MAIN13 = WIDGET_BASE(GROUP_LEADER=Group, $
      ROW=5, $
      MAP=1, $
      TITLE='Screen Calibration', $
      UVALUE='MAIN13')

  BASE2 = WIDGET_BASE(MAIN13, $
      COLUMN=5, $
      FRAME=2, $
      MAP=1, $
      TITLE='Menu Base', $
      UVALUE='BASE2')

  calFileMenuDesc = [ $
      { CW_PDMENU_S,       3, 'File' }, $ ;        0
        { CW_PDMENU_S,       0, 'Open ...' }, $ ;        1
        { CW_PDMENU_S,       0, 'Save ...' }, $ ;        2
        { CW_PDMENU_S,       2, 'Quit' } $  ;      3

  ]
  calFilePDMENU = CW_PDMENU(BASE2, calFileMenuDesc, /RETURN_FULL_NAME, $
                       UVALUE='calFilePDMENU')

  BASE3 = WIDGET_BASE(BASE2, $
      COLUMN=1, $
      FRAME=2, $
      MAP=1, $
      TITLE='Buttons', $
      UVALUE='BASE3')

  setCalBUTTON = WIDGET_BUTTON( BASE3, $
      UVALUE='setCalBUTTON', $
      VALUE='Set Calibration Now')

  updateBUTTON = WIDGET_BUTTON( BASE3, $
      UVALUE='updateBUTTON', $
      VALUE='Update Image')

  BASE4 = WIDGET_BASE(BASE2, $
      COLUMN=1, $
      FRAME=2, $
      MAP=1, $
      TITLE='Buttons', $
      UVALUE='BASE4')

  selRectDesc = [ $
    'Set CAL Points (Dashed)', $
    'Set ROI Rectangle (Solid)' ]

  selRectBGROUP = CW_BGROUP( BASE4, selRectDesc, $
      ROW=2, $
      EXCLUSIVE=1, $
      SET_VALUE = 0,  $
      UVALUE='selRectBGROUP')

  plotChoiceDesc = [ $
    'Warp Image' ] 
  plotChoiceBGROUP = CW_BGROUP( BASE2, plotChoiceDesc, $
      ROW=1, $
      NONEXCLUSIVE=1, $
      UVALUE='plotChoiceBGROUP')

  BASE7 = WIDGET_BASE(MAIN13, $
      ROW=1, $
      MAP=1, $
      TITLE='Message Text Top', $
      UVALUE='BASE7')

  TextVal110 = [ $
    '<< Select Screen # >>' ]
  TEXT8 = WIDGET_TEXT( BASE7,VALUE=TextVal110, $
      UVALUE='TEXT8', $
      XSIZE=80, $
      YSIZE=1)


  BASE9 = WIDGET_BASE(MAIN13, $
      COLUMN=1, $
      MAP=1, $
      TITLE='Image Area', $
      UVALUE='BASE9')

  LABEL61 = WIDGET_LABEL( BASE9, $
      VALUE='CAL Box: LMB-Move, MMB-Resize, RMB-Enter')

  DRAW10 = WIDGET_DRAW( BASE9, $
      BUTTON_EVENTS=1, $
      RETAIN=2, $
      UVALUE='DRAW10', $
      XSIZE=512, $
      YSIZE=480)

  BASE24 = WIDGET_BASE(MAIN13, $
      COLUMN=3, $
      FRAME=2, $
      MAP=1, $
      TITLE='Cal Entry Base', $
      UVALUE='BASE24')

  TL_BASE = WIDGET_BASE(BASE24, $
      ROW=2, $
      FRAME=2, $
      MAP=1, $
      TITLE='T/L Entry', $
      UVALUE='TL_BASE')

  TL_LABEL = WIDGET_LABEL( TL_BASE, $
      UVALUE='TL_LABEL', $
      VALUE='Top/Left Coordinates')

  TL_ENTRY_BASE = WIDGET_BASE(TL_BASE, $
      COLUMN=3, $
      MAP=1, $
      TITLE='entry', $
      UVALUE='TL_ENTRY_BASE')

  TL_X_FIELD = CW_FIELD( TL_ENTRY_BASE, $
      VALUE=imageData.calTemp.tl_x_EGU, $
      ROW=1, $
      FLOAT=1, $
      RETURN_EVENTS=1, $
      TITLE='X:', $
      UVALUE='TL_X_FIELD', $
      XSIZE=6)

  TL_Y_FIELD = CW_FIELD( TL_ENTRY_BASE, $
      VALUE=imageData.calTemp.tl_y_EGU, $
      ROW=1, $
      FLOAT=1, $
      RETURN_EVENTS=1, $
      TITLE='Y:', $
      UVALUE='TL_Y_FIELD', $
      XSIZE=6)

  TL_BUTTON = WIDGET_BUTTON( TL_ENTRY_BASE, $
      UVALUE='TL_BUTTON', $
      VALUE='Set')

  BL_BASE = WIDGET_BASE(BASE24, $
      ROW=2, $
      FRAME=2, $
      MAP=1, $
      TITLE='B/L Entry', $
      UVALUE='BL_BASE')

  BL_LABEL = WIDGET_LABEL( BL_BASE, $
      UVALUE='BL_LABEL', $
      VALUE='Bottom/Left Coordinates')

  BL_ENTRY_BASE = WIDGET_BASE(BL_BASE, $
      COLUMN=3, $
      MAP=1, $
      TITLE='entry', $
      UVALUE='BL_ENTRY_BASE')

  BL_X_FIELD = CW_FIELD( BL_ENTRY_BASE, $
      VALUE=imageData.calTemp.bl_x_EGU, $
      ROW=1, $
      FLOAT=1, $
      RETURN_EVENTS=1, $
      TITLE='X:', $
      UVALUE='TL_X_FIELD', $
      XSIZE=6)

  BL_Y_FIELD = CW_FIELD( BL_ENTRY_BASE, $
      VALUE=imageData.calTemp.bl_y_EGU, $
      ROW=1, $
      FLOAT=1, $
      RETURN_EVENTS=1, $
      TITLE='Y:', $
      UVALUE='TL_Y_FIELD', $
      XSIZE=6)

  BL_BUTTON = WIDGET_BUTTON( BL_ENTRY_BASE, $
      UVALUE='BL_BUTTON', $
      VALUE='Set')

  TR_BASE = WIDGET_BASE(BASE24, $
      ROW=2, $
      FRAME=2, $
      MAP=1, $
      TITLE='T/R Entry', $
      UVALUE='TR_BASE')

  TR_LABEL = WIDGET_LABEL( TR_BASE, $
      UVALUE='TR_LABEL', $
      VALUE='Top/Right Coordinates')

  TR_ENTRY_BASE = WIDGET_BASE(TR_BASE, $
      COLUMN=3, $
      MAP=1, $
      TITLE='entry', $
      UVALUE='BR_ENTRY_BASE')

  TR_X_FIELD = CW_FIELD( TR_ENTRY_BASE, $
      VALUE=imageData.calTemp.tr_x_EGU, $
      ROW=1, $
      FLOAT=1, $
      RETURN_EVENTS=1, $
      TITLE='X:', $
      UVALUE='TR_X_FIELD', $
      XSIZE=6)

  TR_Y_FIELD = CW_FIELD( TR_ENTRY_BASE, $
      VALUE=imageData.calTemp.tr_y_EGU, $
      ROW=1, $
      FLOAT=1, $
      RETURN_EVENTS=1, $
      TITLE='Y:', $
      UVALUE='TR_Y_FIELD', $
      XSIZE=6)

  TR_BUTTON = WIDGET_BUTTON( TR_ENTRY_BASE, $
      UVALUE='TR_BUTTON', $
      VALUE='Set')

  BR_BASE = WIDGET_BASE(BASE24, $
      ROW=2, $
      FRAME=2, $
      MAP=1, $
      TITLE='B/R Entry', $
      UVALUE='BR_BASE')

  BR_LABEL = WIDGET_LABEL( BR_BASE, $
      UVALUE='BR_LABEL', $
      VALUE='Bottom/Right Coordinates')

  BR_ENTRY_BASE = WIDGET_BASE(BR_BASE, $
      COLUMN=3, $
      MAP=1, $
      TITLE='entry', $
      UVALUE='BR_ENTRY_BASE')

  BR_X_FIELD = CW_FIELD( BR_ENTRY_BASE, $
      VALUE=imageData.calTemp.br_x_EGU, $
      ROW=1, $
      FLOAT=1, $
      RETURN_EVENTS=1, $
      TITLE='X:', $
      UVALUE='BR_X_FIELD', $
      XSIZE=6)

  BR_Y_FIELD = CW_FIELD( BR_ENTRY_BASE, $
      VALUE=imageData.calTemp.br_y_EGU, $
      ROW=1, $
      FLOAT=1, $
      RETURN_EVENTS=1, $
      TITLE='Y:', $
      UVALUE='BR_Y_FIELD', $
      XSIZE=6)

  BR_BUTTON = WIDGET_BUTTON( BR_ENTRY_BASE, $
      UVALUE='BR_BUTTON', $
      VALUE='Set')

  BASE25 = WIDGET_BASE(BASE24, $
      COLUMN=1, $
      FRAME=2, $
      MAP=1, $
      UVALUE='BASE25')

  rotMenuDesc = [ $
      { CW_PDMENU_S,       3, 'Rotation' }, $ ;        0
        { CW_PDMENU_S,       0, '0' }, $ ;        1
        { CW_PDMENU_S,       0, '90' }, $ ;        2
        { CW_PDMENU_S,       0, '180' }, $ ;        3
        { CW_PDMENU_S,       2, '270' } $  ;      4

  ]
  rotPDMENU = CW_PDMENU(BASE25, rotMenuDesc, /RETURN_FULL_NAME, $
      UVALUE='rotPDMENU')

  eguFIELD = CW_FIELD( BASE25, $
      VALUE=imageData.calTemp.egu, $
      COLUMN=1, $
      STRING=1, $
      RETURN_EVENTS=1, $
      TITLE='EGU', $
      UVALUE='eguFIELD', $
      XSIZE=6)

  descFIELD = CW_FIELD( BASE24, $
      VALUE=imageData.calTemp.desc, $
      COLUMN=1, $
      STRING=1, $
      RETURN_EVENTS=1, $
      TITLE='DESCRIPTION', $
      UVALUE='descFIELD', $
      XSIZE=20)

  setUnCalBUTTON = WIDGET_BUTTON( MAIN13, $
      UVALUE='setUnCalBUTTON', $
      VALUE='Set To Uncalibrated')

  ; For now, only expect rectangular fiducials
  WIDGET_CONTROL, BL_X_FIELD, SENSITIVE=0
  WIDGET_CONTROL, BL_Y_FIELD, SENSITIVE=0
  WIDGET_CONTROL, TR_X_FIELD, SENSITIVE=0
  WIDGET_CONTROL, TR_Y_FIELD, SENSITIVE=0
  WIDGET_CONTROL, TL_BUTTON, SENSITIVE=0
  WIDGET_CONTROL, TR_BUTTON, SENSITIVE=0
  WIDGET_CONTROL, BL_BUTTON, SENSITIVE=0
  WIDGET_CONTROL, BR_BUTTON, SENSITIVE=0
  WIDGET_CONTROL, plotChoiceBGROUP, SENSITIVE=0

  WIDGET_CONTROL, MAIN13, /REALIZE

  ; Get drawable window index

  COMMON DRAW10_Comm, DRAW10_Id, TEXT8_Id, selRectBGROUP_Id, calPointEntry
  WIDGET_CONTROL, DRAW10, GET_VALUE=DRAW10_Id
  TEXT8_Id = TEXT8
  selRectBGROUP_Id = selRectBGROUP

  calPointEntry = 0L

  COMMON SC_WIDGETS, sc_widget_ids

  sc_widget_ids = {                    $
      tl_x          : TL_X_FIELD,   $ 
      tl_y          : TL_Y_FIELD,   $
      bl_x          : BL_X_FIELD,   $ 
      bl_y          : BL_Y_FIELD,   $ 
      tr_x          : TR_X_FIELD,   $
      tr_y          : TR_Y_FIELD,   $
      br_x          : BR_X_FIELD,   $
      br_y          : BR_Y_FIELD    $
  }


  str1 = imageData.calTemp.desc + '    Rotation = ' +  $
         STRTRIM(STRING(imageData.calTemp.rotation),2)
  IF(imageData.calTemp.calibrated NE 0) THEN str1 = str1 + '  CALIBRATED' $
  ELSE str1 = str1 + '  UNCALIBRATED'
  WIDGET_CONTROL, TEXT8_Id, SET_VALUE=str1

  DRAW_IMAGE

  XMANAGER, 'MAIN13', MAIN13

END

PRO UPDATE_COORD

  COMMON CATCH_IMAGE, widget_ids, imageData
  COMMON SC_WIDGETS, sc_widget_ids

  WIDGET_CONTROL, sc_widget_ids.tl_x, SET_VALUE=imageData.calTemp.tl_x_EGU
  WIDGET_CONTROL, sc_widget_ids.tl_y, SET_VALUE=imageData.calTemp.tl_y_EGU
  WIDGET_CONTROL, sc_widget_ids.bl_x, SET_VALUE=imageData.calTemp.bl_x_EGU
  WIDGET_CONTROL, sc_widget_ids.bl_y, SET_VALUE=imageData.calTemp.bl_y_EGU
  WIDGET_CONTROL, sc_widget_ids.tr_x, SET_VALUE=imageData.calTemp.tr_x_EGU
  WIDGET_CONTROL, sc_widget_ids.tr_y, SET_VALUE=imageData.calTemp.tr_y_EGU
  WIDGET_CONTROL, sc_widget_ids.br_x, SET_VALUE=imageData.calTemp.br_x_EGU
  WIDGET_CONTROL, sc_widget_ids.br_y, SET_VALUE=imageData.calTemp.br_y_EGU
  

END
